setwd("C:\\Users\\cornelia\\Documents\\IBS workship Hannover 2017\\CaseStudyBOWH")

covey<-read.csv("covey.csv")
source("Functions.r")
head(covey)
max(covey$distance,na.rm=T)
# creating a data object which can be used with the functions below
covey.data<-create.data(covey,500,sampler="points","m",binned=F,
  dis.cov=c("Type"),count.cov=c("Type","JDctr"),
  dis.factor.cov = c("Type"), count.factor.cov = c("Type"),
  conditional = T)

# loading the data object covey.data instead of creating it with create.data()
load("covey.data.rdata")

# to have a look at what covey.data contains
str(covey.data, max.level=1)

# a quick peek at the data
head(covey.data$dis.object)

head(covey.data$glmm.data)

###########################
# Exercise 1
# this function is loaded into your workspace using the source command above
f.haz.function.pt<-function(dis,sigma,shape) {
  f <- dis*(1-exp(-(dis/sigma)^(-shape)))
  f
}

# This function is loaded into your workspace using the source command above
covey.hr.detmodel.log.lik<-function(p, datax){
  ## Part 1: setting up the parameter values p for covariates 
  # the detection model
  scale<-exp(p[1])     # the scale intercept 
  shape<-exp(p[2])    # the shape parameter on the log scale
  ## Part 2: the likelihood component pertaining to the detection model
  # calculating the f(y) for each observed distances
  le<-nrow(datax$dis.object)
  fe<-numeric(le)
  # calculating the f(y) for each observation 
  norm.const<-integrate(f.haz.function.pt,0,datax$w,scale,shape)$value
  for (e in 1:le){
  fe[e]<-f.haz.function.pt(datax$dis.object$distance[e],scale,shape)/norm.const
  }
# the sum of the log(f(y))
log.e<-sum(log(fe))
return(log.e)
}

# lower and upper limits are given on the un-transformed scale for both parameters
lolim<-c(50,1)
uplim<-c(1000,10)

# This function is uploaded into your workspace using the source command above
l.prior<-function(coef,lo,up){
  l.u.int<-log(dunif(coef,lo,up))
  if(any(abs(l.u.int)==Inf))l.u.int<- -100000
  sum(l.u.int)
}

############# proposal distributions
# Standard deviations for normal proposal distributions for detection function parameters
q.sd<-c(3.5,0.1)

## initial values for the detection function parameters
# scale intercept and shape (both enter the model on the log scale)
sig.0<-log(130) # the scale intercept
sha.0<-log(2.5) # shape parameter

# combining initial values in a single array for input into the likelihood function 
curparam<-c(sig.0,sha.0)

# number of iterations for the MC chain
nt<-20000

# param.mat holds the values for all parameters throughout the chain
param.mat.ex1<-matrix(NA,nt,length(curparam))
param.mat.ex1<-data.frame(param.mat.ex1)
names(param.mat.ex1)<-c("sig","sha")
param.mat.ex1[1,]<-curparam

########################## Metropolis Hastings updating algorithm ############ 
# nt is the number of iterations and is set above
# Row 1 in param.mat.ex1 contains the initial values, so we start with i = 2
set.seed(1234)
#t1<-unclass(Sys.time())
for (i in 2:nt){
  print(i)
  
  # updating the model parameters
  newparam<-curparam  
  for (b in c(1:2)){
    num<-NA
    den<-NA
    # proposing to update the bth parameter
    u<-rnorm(1,0,q.sd[b]) 
    # the scale and shape parameters are on the log scale in curparam and newparam
    # the boundaries for the priors and the proposal densities are not on the log scale
    newparam[b]<-log(exp(curparam[b])+u)
    new.l.prior<-l.prior(exp(newparam[b]),lolim[b],uplim[b])
    cur.l.prior<-l.prior(exp(curparam[b]),lolim[b],uplim[b])
    num<-covey.hr.detmodel.log.lik(newparam,covey.data) + new.l.prior
    den<-covey.hr.detmodel.log.lik(curparam,covey.data) + cur.l.prior
    A<-min(1,exp(num-den))
    V<-runif(1)
    ifelse(V<=A,curparam[b]<-newparam[b],newparam[b]<-curparam[b])    
  }
  
  # storing the updated parameter values
  param.mat.ex1[i,]<-curparam
  
  # saving the parameter matrices every 5000 iterations
  if(!is.na(match(i,seq(0,nt,5000))==T)){
    save(param.mat.ex1,file='param.mat.ex1.RData')
  }
}

# The following code will only work if the MH algorithm has run for several iterations. 
# We recommend running at least 100 iterations. 
# producing trace plots for all parameters for iterations 1:i 
par(mfrow=c(1,2))
for (b in 1:2){
plot(param.mat.ex1[1:i,b],t='l',xlab="Iteration",ylab="")
if(b == 1) {title(main = expression("Parameter " * sigma[hr]))  }
if(b == 2) {title(main = expression("Parameter " * tau))  }
  }

load("param.mat.ex1.rdata")

# producing trace plots for all parameters for iterations 1:nt 
par(mfrow=c(1,2))
for (b in 1:2){
plot(param.mat.ex1[1:nt,b],t='l',xlab="Iteration",ylab="")
if(b == 1) {title(main = expression("Parameter " * sigma[hr]))  }
if(b == 2) {title(main = expression("Parameter " * tau))  }
  }

nt=20000
# means
round(apply(param.mat.ex1[1000:nt,1:2],2,mean),4)

# standard deviations
round(apply(param.mat.ex1[1000:nt,1:2],2,sd),4)

# 95% credible intervals
round(apply(param.mat.ex1[1000:nt,1:2],2,quantile,probs=c(0.025,0.975)),4)

par(mfrow=c(1,2))
for (b in 1:2){
  plot(density(param.mat.ex1[10000:nt,b]),t='l',main="",ylab="Density",xlab="Parameter value")
if(b == 1) {title(main = expression("Parameter " * sigma[hr]))  }
if(b == 2) {title(main = expression("Parameter " * tau))  }
}

efa.ex1<-array(NA,nt)
for (i in 1:nt){
  efa.ex1[i]<-integrate(f.haz.function.pt,0,covey.data$w,exp(param.mat.ex1[i,1]),exp(param.mat.ex1[i,2]))$value*pi*2
}

par(mfrow=c(1,2))
plot(density(efa.ex1[1000:nt]/(1000*1000)),main="Effective Area",xlab=expression(km^2))
plot(efa.ex1[1000:nt]/(1000*1000),main="Effective Area",ylab=expression(km^2),xlab="Iteration",t='l')

# means
round(mean(efa.ex1[1000:nt]/(1000*1000)),4)

# standard deviations
round(sd(efa.ex1[1000:nt]/(1000*1000)),4)

# 95% credible intervals
round(quantile(efa.ex1[1000:nt]/(1000*1000),probs=c(0.025,0.975)),4)

# The hazard-rate detection function
g.haz.function.pt<-function(dis,sigma,shape) {
  g <- 1-exp(-(dis/sigma)^(-shape))
  g
}


par(mfrow=c(1,2))
hist(covey.data$dis.object$distance,freq = F,xlab="m",main="PDF of observed distances",ylab="f(y)")
scale = mean(exp(param.mat.ex1[1000:nt,1]))
shape = mean(exp(param.mat.ex1[1000:nt,2]))
norm.const<-integrate(f.haz.function.pt,0,covey.data$w,scale,shape)$value
lines(f.haz.function.pt(seq(0,500,1),scale,shape)/norm.const~seq(0,500,1),col=2,lwd=2)

plot(g.haz.function.pt(seq(0,500,1),scale,shape)~seq(0,500,1),type='l',xlab="m",ylab="g(y)",main="Detection function")


###########################
# Exercise 2
# This function is loaded into your workspace using the source command above
covey.int.log.lik<-function(p, raneff, datax){
  ## Part 1: setting up the parameter values p for the detection function and count models 
  # the detection model
  scale<-exp(p[1])  # the scale intercept 
  shape<-exp(p[2])  # the shape parameter on the log scale
  # the count model 
  int<-p[3]         # the intercept
  typ<-c(0,p[4])    # coefficient for Type level "TREAT" (level "CONTROL" is absorbed in the intercept)
  sd.ran<-exp(p[5]) # the random effect standard deviation on the log scale

# Part 2: the likelihood component pertaining to the detection model
# calculating the f(y) for each observed distances
le<-nrow(datax$dis.object)
fe<-numeric(le)
# calculating the f(y) for each observation (note that the truncation distance is stored in datax$w)
norm.const<-integrate(f.haz.function.pt,0,datax$w,scale,shape)$value
for (e in 1:le){
  fe[e]<-f.haz.function.pt(datax$dis.object$distance[e],scale,shape)/norm.const
}
# the sum of the log(f(y))
log.e<-sum(log(fe))

# Part 3: the likelihood component pertaining to the count model  
# setting up indices for the factor covariate Type and random effect coefficients
Type0<-match(datax$glmm.data$Type,sort(unique(datax$glmm.data$Type)))
gr.id<-sort(unique(datax$glmm.data$gr.id))
Ran0<-match(datax$glmm.data$gr.id,gr.id)
# log of effective area (here: same for each visit to a point due to global det model)
l.efa<-log(norm.const*pi*2)
# calculate the log-Poisson density for each count 
lambda<-exp(int + typ[Type0] + raneff[Ran0] + l.efa)
dpois.y<-dpois(datax$glmm.data$detections,lambda)
logdpois.y<-sum(log(dpois.y))
# calculate the log of the normal density for each random effect coefficient
log.dnorm.raneff<-sum(log(dnorm(raneff,0,sd.ran)))
# adding up the likelihood components
log.lik<-logdpois.y + log.dnorm.raneff + log.e
# the function return
return(log.lik)
}

# Note: lower and upper limits are given on the un-transformed scale for all parameters including scale, shape and sd.ran
lolim<-c(50,1,rep(-5,2),0.0001)
uplim<-c(1000,10,rep(5,2),3)

# This function is uploaded into your workspace using the source command above
l.prior<-function(coef,lo,up){
  l.u.int<-log(dunif(coef,lo,up))
  if(any(abs(l.u.int)==Inf))l.u.int<- -100000
  sum(l.u.int)
}

############# proposal distributions
# Standard deviations for normal proposal distributions for all parameters: 
# detection function (1:6) and count model (7:13) including random effect standard deviation
q.sd<-c(3.5,0.1,0.02,0.03,0.02)

## initial values for the detection function parameters
# scale intercept and shape (enter the model on the log scale)
sig.0<-log(130) # the scale intercept
sha.0<-log(2.5) # shape parameter

## initial values for count model parameters
int.0<- -13     # intercept

# type coefficient
typ.0<- 0.4     # level treat

# initial values for random effect standard deviation and coefficients
sd.ran.0<-log(1)
j=length(unique(covey.data$glmm.data$gr.id)) # number of sites in covey data 
set.seed(1234)
raneff<-rnorm(j,0,exp(sd.ran.0))

# combining the initial values in a single array 
curparam<-c(sig.0,sha.0,int.0,typ.0,sd.ran.0)

# number of iterations for the MC chain
nt<-20000

# param.mat holds the values for all parameters throughout the chain
param.mat.ex2<-matrix(NA,nt,length(curparam))
param.mat.ex2<-data.frame(param.mat.ex2)
names(param.mat.ex2)<-c("sig","sha","int","typ","sd.ran")
param.mat.ex2[1,]<-curparam
# raneff.mat holds the values for all random effect coefficients throughout the chain
raneff.mat.ex2<-matrix(NA,nt,j)
raneff.mat.ex2<-data.frame(raneff.mat.ex2)
raneff.mat.ex2[1,]<-raneff

covey.int.log.lik.raneff.update<-function(p,raneff,datax){
  # setting up the covariates
  scale<-exp(p[1])
  shape<-exp(p[2])
  int<-p[3]
  typ<-c(0,p[4])
  sd.ran<-exp(p[5])
  
  # log of effective area (here: same for each visit to a point due to global det model)
  norm.const<-integrate(f.haz.function.pt,0,datax$w,scale,shape)$value
  l.efa<-log(norm.const*pi*2)
  # setting up indices for the factor covariate Type and random effect coefficients
  Type0<-match(datax$glmm.data$Type,sort(unique(datax$glmm.data$Type)))
  gr.id<-sort(unique(datax$glmm.data$gr.id))
  Ran0<-match(datax$glmm.data$gr.id,gr.id)
  # calculate the log-Poisson density for each count 
  lambda<-exp(int + (typ[Type0]) + raneff[Ran0] + l.efa)
  #lambda<-as.numeric(unlist(lambda))
  # update each coefficient 
  for (r in 1:length(gr.id)){
    u<-rnorm(1,0,0.2)
    yr<-which(datax$glmm.data$gr.id==gr.id[r])
    dpois.y.new<-dpois(datax$glmm.data$detections[yr],lambda[yr]*exp(u))
    num<-sum(log(dpois.y.new))+log(dnorm(raneff[r]+u,0,sd.ran))
    dpois.y.cur<-dpois(datax$glmm.data$detections[yr],lambda[yr])
    den<-sum(log(dpois.y.cur))+log(dnorm(raneff[r],0,sd.ran))
    A<-min(1,exp(num-den))
    V<-runif(1)
    if(V<=A){raneff[r]<-raneff[r]+u}
  }
  # return updated coefficients
  raneff
}

########################## Metropolis Hastings updating algorithm ############ 
# nt is the number of iterations and is set above
# row 1 in param.mat.ex2 and raneff.mat.ex2 contains the initial values, so we start with i = 2
set.seed(1234)
for (i in 2:nt){
  print(i)
  
  # updating the 5 model parameters
  newparam<-curparam  
  for (b in c(1:5)){
    num<-NA
    den<-NA
    # proposing to update the bth parameter
    u<-rnorm(1,0,q.sd[b]) 
    # the scale intercept, shape and RE sd are on the log scale in curparam and newparam while the boundaries for the priors are not on the log scale
    if(!is.na(match(b,c(1,2,5)))){
      if((exp(curparam[b])+u)>=lolim[b]){
      newparam[b]<-log(exp(curparam[b])+u)
      new.l.prior<-l.prior(exp(newparam[b]),lolim[b],uplim[b])
      cur.l.prior<-l.prior(exp(curparam[b]),lolim[b],uplim[b])
    num<-covey.int.log.lik(newparam,raneff,covey.data) + new.l.prior
    den<-covey.int.log.lik(curparam,raneff,covey.data) + cur.l.prior
    A<-min(1,exp(num-den))
    V<-runif(1)
    ifelse(V<=A,curparam[b]<-newparam[b],newparam[b]<-curparam[b])    
      }
    }
    else{
      newparam[b]<-curparam[b]+u
      new.l.prior<-l.prior(newparam[b],lolim[b],uplim[b])
      cur.l.prior<-l.prior(curparam[b],lolim[b],uplim[b])
    num<-covey.int.log.lik(newparam,raneff,covey.data) + new.l.prior
    den<-covey.int.log.lik(curparam,raneff,covey.data) + cur.l.prior
    A<-min(1,exp(num-den))
    V<-runif(1)
    ifelse(V<=A,curparam[b]<-newparam[b],newparam[b]<-curparam[b])    
    }
    num<-covey.int.log.lik(newparam,raneff,covey.data) + new.l.prior
    den<-covey.int.log.lik(curparam,raneff,covey.data) + cur.l.prior
    A<-min(1,exp(num-den))
    V<-runif(1)
    ifelse(V<=A,curparam[b]<-newparam[b],newparam[b]<-curparam[b])    
  }
  
  # storing the updated parameter values
  param.mat.ex2[i,]<-curparam
  
  # updating the random effect coefficients
  raneff<-covey.int.log.lik.raneff.update(curparam, raneff, covey.data)
  # storing the updated random effect coefficients
  raneff.mat.ex2[i,]<-raneff
  
  # saving the parameter matrices every 5000 iterations
  if(!is.na(match(i,seq(0,nt,5000))==T)){
    save(param.mat.ex2,file='param.mat.ex2.RData')
    save(raneff.mat.ex2,file='raneff.mat.ex2.RData')
  }
}# end of i loop

# The following code will only work if the MH algorithm has run for several iterations. 
# We recommend running at least 100 iterations. 
# producing trace plots for each of the 5 parameters for iterations 1:i 
par(mfrow=c(3,2))
for (b in 1:5){
plot(param.mat.ex2[1:i,b],t='l',xlab="Iteration",main="",ylab="")
if(b == 1) {title(main = expression("Parameter " * sigma[hr]))}
if(b == 2) {title(main = expression("Parameter " * tau))}
if(b == 3) {title(main = expression("Parameter " * beta[0]))}
if(b == 4) {title(main = expression("Parameter " * beta[1]))}
if(b == 5) {title(main = expression("Parameter " * sigma[l[hr]]))}
  }

load("param.mat.ex2.rdata")
load("raneff.mat.ex2.rdata")


# producing trace plots for all parameters for iterations 1:nt 
par(mfrow=c(3,2))
for (b in 1:5){
plot(param.mat.ex2[1:nt,b],t='l',xlab="Iteration",ylab="")
if(b == 1) {title(main = expression("Parameter " * sigma[hr]))  }
if(b == 2) {title(main = expression("Parameter " * tau))  }
if(b == 3) {title(main = expression("Parameter " * beta[0]))}
if(b == 4) {title(main = expression("Parameter " * beta[1]))}
if(b == 5) {title(main = expression("Parameter " * sigma[l[hr]]))}
  }

nt=20000
# means
round(apply(param.mat.ex2[1000:nt,1:5],2,mean),4)

# standard deviations
round(apply(param.mat.ex2[1000:nt,1:5],2,sd),4)

# 95% credible intervals
round(apply(param.mat.ex2[1000:nt,1:5],2,quantile,probs=c(0.025,0.975)),4)

par(mfrow=c(3,2))
for (b in 1:5){
  plot(density(param.mat.ex2[1000:nt,b]),t='l',main="",ylab="Density",xlab="Parameter value")
if(b == 1) {title(main = expression("Parameter " * sigma[hr]))  }
if(b == 2) {title(main = expression("Parameter " * tau))  }
if(b == 3) {title(main = expression("Parameter " * beta[0]))}
if(b == 4) {title(main = expression("Parameter " * beta[1]))}
if(b == 5) {title(main = expression("Parameter " * sigma[l[hr]]))}
}

Dbl.ex2<-array(NA,nt)
# calculate the baseline density
# omit the burn-in
for (i in 1000:nt){
  Dbl.ex2[i]<-exp(param.mat.ex2[i,3]+(0.5*exp(param.mat.ex2[i,5])^2))
}
Dbl.ex2<-Dbl.ex2*1000*1000

# the posterior mean, standard deviation and central credible interval for  
# baseline density after omitting the burn-in
# (rounded to 2 decimal points)
round(mean(Dbl.ex2[1000:nt]),2)

round(sd(Dbl.ex2[1000:nt]),2)

round(quantile(Dbl.ex2[1000:nt],prob=c(0.025,0.975)),2)

###########################
# Exercise 3

# This function is loaded into your workspace using the source command above
covey.rj.int.log.lik<-function(p, raneff, datax){
  ## Part 1: setting up the parameter values p for the detection function and count models 
  # the detection model
  scale<-exp(p[1])     # the scale intercept 
  shape<-exp(p[2])     # the shape parameter on the log scale
  # the count model 
  int<-p[3]            # the intercept
  typ<-c(0,p[4])       # coefficient for Type level "TREAT" (level "CONTROL" is absorbed in the intercept)
  day<-p[5]            # coefficient for JDctr
  sd.ran<-exp(p[6])    # the random effect standard deviation on the log scale

# Part 2: the likelihood component pertaining to the detection model
# calculating the f(y) for each observed distances
le<-nrow(datax$dis.object)
fe<-numeric(le)
# calculating the f(y) for each observation (note that the truncation distance is stored in datax$w)
norm.const<-integrate(f.haz.function.pt,0,datax$w,scale,shape)$value
for (e in 1:le){
  fe[e]<-f.haz.function.pt(datax$dis.object$distance[e],scale,shape)/norm.const
}
# the sum of the log(f(y))
log.e<-sum(log(fe))

# Part 3 of covey.int.log.lik(): the likelihood component pertaining to the count model  
# setting up indices for the factor covariate Type and random effect coefficients
Type0<-match(datax$glmm.data$Type,sort(unique(datax$glmm.data$Type)))
gr.id<-sort(unique(datax$glmm.data$gr.id))
Ran0<-match(datax$glmm.data$gr.id,gr.id)
# log of effective area (here: same for each visit to a point due to global det model)
l.efa<-log(norm.const*pi*2)
# calculate the log-Poisson density for each count 
lambda<-exp(int + typ[Type0] + covey.data$glmm.data$JDctr*day + raneff[Ran0] + l.efa)
dpois.y<-dpois(datax$glmm.data$detections,lambda)
logdpois.y<-sum(log(dpois.y))
# calculate the log of the normal density for each random effect coefficient
log.dnorm.raneff<-sum(log(dnorm(raneff,0,sd.ran)))
# adding up the likelihood components
log.lik<-logdpois.y + log.dnorm.raneff + log.e
# the function return
return(log.lik)
}

# setting the lower and upper limits for the uniform prior density
lolim<-c(50,1,-25,-5,-5,0.0001)
uplim<-c(1000,10,5,5,5,3)

# This function is uploaded into your workspace using the source command above
l.prior<-function(coef,lo,up){
  l.u.int<-log(dunif(coef,lo,up))
  if(any(abs(l.u.int)==Inf))l.u.int<- -100000
  sum(l.u.int)
}

# proposal distributions for parameters in the detection and count model
# for RJ step: proposals of new parameters
prop.mean<-c(270, 3.5, -13, 0.6, .2, 0.5)
prop.sd<-c(1.41, 0.84, 0.1, 0.2, 0.2, 0.3)
# for MH step: proposals to update current parameters
q.sd<-c(3.5, 0.1, 0.02, 0.03, 0.005, 0.02)

# number of iterations in the RJMCMC chain
nt<-20000

# array that will keep track of the model choice (stored as an integer number referring to the model number (row number) from count.list below)
count.model<-array(NA,nt)

# we need to know which model contains which parameters
# row number indicates the model number 
# column names indicate which parameters are switched on or off for the respective model 
# 0: parameter is switched off 
# 1: parameter is switched on
count.list<-matrix(0,2,6)  # 2 count model choices: 
# model 1: int + typ 
# model 2: int + typ + JDctr 
# columns: scale, shape, int, type level TREAT, JDctr, sd.ran 
colnames(count.list)<-c("scale","shape","int","type.tr","JDctr","sd.ran")

# all detection models contain the scale intercept and shape parameter (hazard rate)
# all count models contain the intercept, covariate type and sd.ran
count.list[1,]<-c(rep(1,4),0,1)           # without JDctr 
count.list[2,]<-c(rep(1,6))               # with JDctr 

# data frames for storing the parameter values
param.mat.ex3<-matrix(NA,nt,6)
param.mat.ex3<-data.frame(param.mat.ex3)
# these obtain the same column names as count.list
colnames(param.mat.ex3)<-c("scale","shape","int","type.tr","JDctr","sd.ran")

# data frame for storing the random effect coefficients
j<-length(sort(unique(covey.data$glmm.data$gr.id)))
raneff.mat.ex3<-matrix(NA,nt,j)       # 40 sites
raneff.mat.ex3<-data.frame(raneff.mat.ex3)

## Setting the initial models
# global detection function and intercept + RE sd count model 
curmodel<-1
count.model[1]<-curmodel
# curlist will tell you which parameters are switched on for the current model
curlist<-count.list[curmodel,]

## Setting up initial values
# for detection model: scale intercept and shape (which are both on the log-scale)
sig.0 <- log(130)
sha.0 <- log(2.5)

# initial values for count model parameters
int.0 <- -13        # intercept
typ.0 <- 0.4        # type coefficient
# day.0 <- -0.01    # needed in case you start with model 2
sd.ran.0 <- log(1)  # random effect standard deviation 
j<-length(sort(unique(covey.data$glmm.data$gr.id))) # number of sites
set.seed(1234)
raneff<-rnorm(j,0,exp(sd.ran.0))

# curparam and new param always needs to be of length 6 as the likelihood function covey.rj.int.log.lik expects an array of length 6 for argument p regardless of model
# for model 1 we set the coefficient for JDctr to 0
curparam<-c(sig.0,sha.0,int.0,typ.0,0,sd.ran.0)
param.mat.ex3[1,]<-curparam
raneff.mat.ex3[1,]<-raneff

# nt is the number of iterations and is set above
set.seed(1234)
for (i in 1:nt){
  print(i)
  ########################## RJ step ########################################
  # method: propose to add or delete covariate JDctr depending on whether it is in the current model
  
  newlist<-curlist
  newparam<-curparam
  
  ######################################### count model parameters
  # check if JDctr it is in the current model:
  # propose to add it if it is not in the current model
  b <- 5
  if(curlist[b]==0){
    newlist[b]<-1
    newparam[b]<-rnorm(1,prop.mean[b],prop.sd[b])
    num<-NA
    den<-NA
    # the prior densities
    new.l.prior<-l.prior(newparam[b],lolim[b],uplim[b])
    # likelihood 
    new.lik<-covey.rj.int.log.lik(newparam,raneff,covey.data) 
    cur.lik<-covey.rj.int.log.lik(curparam,raneff,covey.data)
    # proposal density
    prop.dens<-log(dnorm(newparam[b],prop.mean[b],prop.sd[b]))
    # numerator and denominator in acceptance probability
    num<-new.lik+new.l.prior
    den<-cur.lik+prop.dens
  }
  # propose to delete it if it is in the current model
  else{
    newlist[b]<-0  
    newparam[b]<-0
    num<-NA
    den<-NA
    # the prior densities
    cur.l.prior<-l.prior(curparam[b],lolim[b],uplim[b])
    # likelihood 
    new.lik<-covey.rj.int.log.lik(newparam,raneff,covey.data) 
    cur.lik<-covey.rj.int.log.lik(curparam,raneff,covey.data)
    # proposal density
    prop.dens<-log(dnorm(curparam[b],prop.mean[b],prop.sd[b]))
    # numerator and denominator in equation ttt
    num<-new.lik+prop.dens
    den<-cur.lik+cur.l.prior
  }
  A<-min(1,exp(num-den))
  V<-runif(1)
  if(V<=A){
    curparam[b]<-newparam[b]
    curlist[b]<-newlist[b]
  }
  else{
    newparam[b]<-curparam[b]
    newlist[b]<-curlist[b]
  }    
  
  # determining the current model and storing it for this iteration
  curmodel<-match.function(curlist,count.list)
  count.model[i]<-curmodel
  
  ##################################### MH step ###################################
  
  newparam<-curparam  
  # updating current parameters
  for (b in which(curlist==1)){
    num<-NA
    den<-NA
    # proposing to update the bth parameter
    u<-rnorm(1,0,q.sd[b])                
    # remembering that scale, shape and sd.ran are on the log-scale
    if(!is.na(match(b,c(1,2,6)))){
      newparam[b]<-log(exp(curparam[b])+u)
      new.l.prior<-l.prior(exp(newparam[b]),lolim[b],uplim[b])
      cur.l.prior<-l.prior(exp(curparam[b]),lolim[b],uplim[b])
    }
    else{
      newparam[b]<-curparam[b]+u
      new.l.prior<-l.prior(newparam[b],lolim[b],uplim[b])
      cur.l.prior<-l.prior(curparam[b],lolim[b],uplim[b])
    }
    num<-covey.rj.int.log.lik(newparam,raneff,covey.data) + new.l.prior
    den<-covey.rj.int.log.lik(curparam,raneff,covey.data) + cur.l.prior
    A<-min(1,exp(num-den))
    V<-runif(1)
    ifelse(V<=A,curparam[b]<-newparam[b],newparam[b]<-curparam[b])    
  }
  
  param.mat.ex3[i,]<-curparam
  
  # updating the random effect coefficients
  raneff<-covey.rj.int.log.lik.raneff.update(curparam, raneff, covey.data)
  raneff.mat.ex3[i,]<-raneff
  
  # saving the parameter matrices every 5000 iterations
  if(!is.na(match(i,seq(0,nt,5000))==T)){
    save(param.mat.ex3,file='param.mat.ex3.RData')
    save(raneff.mat.ex3,file='raneff.mat.ex3.RData')
    save(count.model,file="model.ex3.RData")
  }
}

load("param.mat.ex3.rdata")
load("model.ex3.rdata")
load("raneff.mat.ex3.rdata")

# to calculate model probabilities (omiting the burn-in)
round(table(count.model[1000:nt])/length(count.model[1000:nt]),6)

plot(count.model,t='l',yaxt="n",ylab="Model",xlab="Iteration")
axis(side=2,at=c(1,2),labels = c(1,2))

# Inference on level "TREAT" for the Type coefficient in the count model
# 1. including all iterations after the burn-in
round(mean(param.mat.ex3[1000:nt,4]),2)

round(sd(param.mat.ex3[1000:nt,4]),2)

round(quantile(param.mat.ex3[1000:nt,4],probs=c(0.025,0.975)),2)

# 2. using model 2 only 
x2<-which(count.model[1000:nt]==2)+999
round(mean(param.mat.ex3[x2,4]),2)

round(sd(param.mat.ex3[x2,4]),2)

round(quantile(param.mat.ex3[x2,4],probs=c(0.025,0.975)),2)


